/*
 *      RT24conv
 *
 *      by Marcus Grber 2000
 *
 *      This program converts Voxware RT24 WAV files from various codecs into
 *      a variant suitable for playback on the 9110. Note that the 9110 cannot
 *      properly play back files larger than 64k (~3.5 min).
 */

#include <stdio.h>
#include <string.h>

/*
 * Nokia 9110 compatible header for RT24 file. "fact" and "data" chunks have
 * to be appended to this.
 */
unsigned char head[] = {
0x52,0x49,0x46,0x46,0x00,0x00,0x00,0x00,0x57,0x41,0x56,0x45,0x66,0x6D,0x74,0x20,
0x22,0x00,0x00,0x00,0x74,0x00,0x01,0x00,0x40,0x1F,0x00,0x00,0x2C,0x01,0x00,0x00,
0x1B,0x00,0x10,0x00,0x10,0x00,0x56,0x4F,0x58,0x2E,0x08,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x56,0x4F,0x58,0x61
};

char buf[1024];

int main(int argc, char **argv)
{
        FILE *fin, *fout;
        unsigned char blk[4];
        long size, inhead[3];
        size_t n;
        int ret = 0;
        struct {
          unsigned      wFormatTag;
          unsigned      nChannels;
          unsigned long nSamplesPerSec;
          unsigned long nAvgBytesPerSec;
          unsigned      nBlockAlign;
          unsigned      wBitsPerSample;
          unsigned      cbSize;
        } WAVEfmt;

        if(argc<3)
        {
          puts("Syntax: RT24CONV <Netscape_RT24>.wav <Output>.wav\n\n"
               "This program converts Voxware RT24 WAV files from various codecs into a\n"
               "variant suitable for playback on the 9110. Note that the 9110 cannot\n"
               "properly play back files larger than 64k (~3.5 min).");
          return 255;
        }

        fin = fopen(argv[1], "rb");
        if(!fin)
        {
          puts("Can't open input file.");
          return 1;
        }

        fout = fopen(argv[2], "wb");
        if(!fout)
        {
          puts("Can't open output file.");
          fclose(fin);
          return 2;
        }

        fwrite(head, sizeof(head), 1, fout);

        fread(inhead, sizeof(inhead), 1, fin);
        if(inhead[0] != 0x46464952 || inhead[2] !=0x45564157)
        {
          puts("Not a WAV file.");
          ret = 3;
        }

        while(!feof(fin) && !ret)
        {
          if(fread(blk, sizeof(blk), 1, fin)!=1 ||
             fread(&size, sizeof(size), 1, fin)!=1)
            break;

          printf("%c%c%c%c: %ld bytes\n", blk[0], blk[1], blk[2], blk[3], size);

          if(strncmp(blk, "fact", 4)==0 || strncmp(blk, "data", 4)==0)
          {
            fwrite(blk, sizeof(blk), 1, fout);
            fwrite(&size, sizeof(size), 1, fout);
            if(size & 1)
              size++;
            while(size)
            {
              n = (size<sizeof(buf))? ((size_t)size) : sizeof(buf);
              fread(buf, 1, n, fin);
              fwrite(buf, 1, n, fout);
              size -= n;
            }
          }
          else if(strncmp(blk, "fmt ", 4)==0)
          {
            fread(&WAVEfmt, sizeof(WAVEfmt), 1, fin);
            fseek(fin, size - sizeof(WAVEfmt), SEEK_CUR);
            printf(" Format tag: %04X\n %d channels @ %ld Hz (%ld Bytes/s)\n",
              WAVEfmt.wFormatTag, WAVEfmt.nChannels, WAVEfmt.nSamplesPerSec,
              WAVEfmt.nAvgBytesPerSec);

            /*
             * Test against known variants of RT24 WAV files.
             *
             * According to the MMREG.H file from the Windows SDK, 0x74 is
             * the manufacturer ID of Voxware rather than a valid WAVE_FORMAT
             * token. Anyway, this value is used by the 9110 when it creates
             * a file from a recording, so I keep it in to make sure that the
             * program can re-process its own output.
             *
             * Not supported: 0x0069 (RT24 Bitstream, non-file)
             */
            if((WAVEfmt.wFormatTag != 0x0074 &&
                  // 9110-generated file
                WAVEfmt.wFormatTag != 0x181C &&
                  // Codec: "Netscape Compatible RT24" or "Voxware RT24"
                WAVEfmt.wFormatTag != 0x0062) ||
                  // Code: "Voxware Codecs"
               WAVEfmt.nChannels != 1 || WAVEfmt.nSamplesPerSec != 8000 ||
               WAVEfmt.nAvgBytesPerSec != 300)
            {
              puts("\nUnsupported format.");
              ret = 3;
            }
          }
          else
          {
            fseek(fin, size, SEEK_CUR);
          }
        }

        size = ftell(fout) - 8;

        fseek(fout, 4, SEEK_SET);
        fwrite(&size, sizeof(size), 1, fout);

        fclose(fout);
        fclose(fin);

        if(ret)                         // Error: remove output file
          unlink(argv[2]);

        return ret;
}
